package main

import "fmt"

// 空接口
type Empty interface {
}

// 空结构体
type EmptyStruct struct {
}

// 接收任意类型的可变参数
func test(args ...interface{}) {
	for i, arg := range args {
		fmt.Println(i, ":", arg)
		switch v := arg.(type) {
		case int:
			fmt.Println("int", v)
		case string:
			fmt.Println("string", v)
		case bool:
			fmt.Println("bool", v)
		default:
			fmt.Printf("%T %#v", v, v)
		}
	}
}

func main() {
	var es EmptyStruct
	es = EmptyStruct{}
	var e Empty
	fmt.Printf("%T %#v %T %#v\n", es, es, e, e) // main.EmptyStruct main.EmptyStruct{} <nil> <nil>

	e = "hello"
	fmt.Println(e)

	e = true
	fmt.Println(e)

	e = es
	fmt.Println(e)

	// 使用一般定义为匿名接口
	var v interface{}
	fmt.Printf("%T %#v\n", v, v)
	v = 1
	fmt.Printf("%T %#v\n", v, v)

	// 调用能接受任意类型、任意长度的函数
	test(1, "hello", false, e)
}
